/* FilterIIRcordic.c infinite input response digital filter using CORDIC fixed point math
 * target: Propeller 2
 * compiler: fastspin

 * author: Nicolas Benezan (nicolas@benezan.de, forum: ManAtWork)
*/

#include <propeller2.h>
#include <stdint.h>

/*
The IIR filters use fixed point math and are optimized for speed.
Care has to be taken to avoid overflows. So the following rules apply:
* coefficients are limited to -2.0 to +2.0
* input signal values should be in the range +/-0x40000000 (31 bits signed)
* be careful with filter types that have gain > 1.0 or 
  tend to overshoot or oscillate

*/

uint32_t CoeffToArcSin (float c)
// We compute the arcsine with the CORDIC solver.
// For this to work x/y has to be on the unit circle (r=1).
{
  int32_t y = c * 1073741824.0f; // 0x40000000 means 1.0
  int32_t h, l, x; 
  __asm {
	abs  y
	shl  y,#1
	qmul y,y
	getqy h
	getqx l
	not  h
	not  l
	qsqrt l,h	// compute x= sqrt(1-y²)
	getqx x
	qvector x,y
	getqy y 	// compute arcsin(y)
  }
  return y;
}

uint32_t CordicIIRfilter (a0, in, a1, x1, a2, x2, a3, x3, b1, y1, b2, y2, b3, y3)
// The CORDIC unit doesn't support signed multiplication.
// We do a trick and compute y = 2*x*sin(arcsin(c/2))
// which is the same as y = x*c for -2 <= c <= +2.
// QROTATE computes y = D * sin(S), x is ignored.
// The arcsine of the coefficients are pre-calculated.
{
  int32_t y0;
  int32_t r;
  __asm {
	qrotate in,a0 // 0 start feeding pipeline
	qrotate x1,a1 // +8 clocks
	qrotate y1,b1 // +16
	qrotate x2,a2 // +24
	qrotate y2,b2 // +32
	qrotate x3,a3 // +40
	qrotate y3,b3 // +48
	getqy   y0    // +56 result of in*a0
	getqy	r     // +64 result of x1*a1
	add	y0,r
	getqy	r     // +72 result of y1*b1
	sub	y0,r
	getqy	r     // +80 result of x2*a2
	add	y0,r
	getqy	r     // +88 result of y2*b2
	sub	y0,r
	getqy	r     // +96 result of x3*a3
	add	y0,r
	getqy	r     // +104 result of y3*b3
	sub	y0,r
  }
}
