/* FilterIIRcordic.h infinite input response digital filter using CORDIC fixed point math
 * target: Propeller 2
 * compiler: fastspin

 * author: Nicolas Benezan (nicolas@benezan.de, forum: ManAtWork)
*/

#ifndef FILTERIIR_H
#define FILTERIIR_H

#include <stdint.h>

/*
The IIR filters use fixed point math and are optimized for speed.
Care has to be taken to avoid overflows. So the following rules apply:
* coefficients are limited to -2.0 to +2.0
* input signal values should be in the range +/-0x40000000 (31 bits signed)
* be careful with filter types that have gain > 1.0 or 
  tend to overshoot or oscillate

*/

#define maxCoeff 4 // order limited to 3 because of CORDIC pipeline

uint32_t CoeffToArcSin (float y) __fromfile ("FilterIIRcordic.c");
uint32_t CordicIIRfilter (a0, in, a1, x1, a2, x2, a3, x3, b1, y1, b2, y2, b3, y3) __fromfile ("FilterIIRcordic.c");

typedef class
{ // private members
  unsigned int n; // number of coefficients = order+1
  int32_t x[maxCoeff]; // input samples
  int32_t y[maxCoeff]; // output samples
  int32_t a[maxCoeff]; // A coefficients
  int32_t b[maxCoeff]; // B coefficients

  // public methods
  void Clear ()
  {
    for (unsigned int i=0; i<maxCoeff; i++)
    {
      x[i]= 0;
      y[i]= 0;
      if (i >= n)
      {
        a[i]= 0; // clear unnecessary coefficients
        b[i]= 0;
      }
    }
  }
  void SetCoefficients (float* ca, float* cb, unsigned int order)
  {
    n= order+1;
    if (n > maxCoeff) n= maxCoeff;
    for (unsigned int i=0; i<n; i++)
    {
      a[i]= CoeffToArcSin (ca[0]);
      b[i]= CoeffToArcSin (cb[i]);
    }
    Clear ();
  }
  float Output (float in)
  {
    for (unsigned int i=n-1; i>0; i--)
    {
      x[i]= x[i-1];
      y[i]= y[i-1];
    }
    x[0]= in;
    y[0]= CordicIIRfilter (a[0], in, a[1], x[1], a[2], x[2], a[3], x[3],
                           b[1], y[1], b[2], y[2], b[3], y[3]);
    /* performs:
    y[0]= a[0] * in;
    for (unsigned int i=1; i<n; i++)
    {
      y[0]+= a[i]*x[i] - b[i]*y[i];
    }
    */
    return y[0];
  }

} iirFilter;

#endif
